function [pathLabels, path] = abstractPath(nba, src, dst, type)
%abstractPath Create path through automaton representing logical path
%
% Solve for a minimum cost accepting cycle
% Ex: (q1,q2), (q2,q3), (q3,q1)
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

A = nba.A;
Aw = nba.Aw;
L = nba.L;

if strcmp(type,'path')
    assert( 1 == length(dst) );
    [d, pred] = shortest_paths(A, src, 'target', dst, 'edge_weight', edge_weight_vector(A, Aw));
    path = path_from_pred(pred, dst);

elseif strcmp(type,'cycle')
    assert( 1 == length(dst) );
    % Augment the graph with the additional state if src == dst
    % TODO: This process is somewhat expensive.
    dst = length(A)+1;
    A(dst,dst) = 0;
    Aw(dst,dst) = 0;
    L{dst,dst} = [];

    for i = 1:dst-1
        A(i,dst) = A(i,src);
        Aw(i,dst) = Aw(i,src);
        L{i,dst} = L{i,src};
    end

    [d, pred] = shortest_paths(A, src, 'target', dst, 'edge_weight', edge_weight_vector(A, Aw));
    path = path_from_pred(pred, dst);
    path(end) = src;

elseif strcmp(type,'to_cycle')
    % Computing shortest path from src to any vertex in dst
    [d, pred] = shortest_paths(A, src, 'edge_weight', edge_weight_vector(A, Aw));
    min_dist = Inf;
    min_vertex = [];
    for i = 1:length(dst)
        if d(dst(i)) < min_dist
            min_dist = d(dst(i));
            min_vertex = dst(i);
        end
    end
    path = path_from_pred(pred, min_vertex);

    if length(path) == 1
        path = [src, src];
    end  
end

pathLabels = getLabels(nba, path);
    
end


function pathLabels = getLabels(nba, path)
%pathLabels Map path to corresponding labels
%
% Output the (safety,reach) pairs along given cycle
% Ex:   (q1,q2): {safe: S, reach: A & S}
%       (q2,q3): {safe: A & B & S, reach: S}
%       (q3,q1): {safe: S, reach: B & S}
%
% :copyright: 2014 by Eric M. Wolff, ewolff@caltech.edu
% :license: GPLv2, see LICENSE for details
  
L = nba.L;
pathLabels = cell(length(path) - 1, 1);

for i = 1:length(pathLabels)        
    safe = L(path(i), path(i));
    reach = L(path(i), path(i+1));
    pathLabels{i} = struct('safe', safe, 'reach', reach);
end
    
end
