function [sol, traj, sys] = trajSim(labelsPre, labelsSuf, N, dt, model, options,...
                                    bigM, ops, blkSafe)
%trajSim Solve constrained reachability problems simultaneously
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details
                                
assert(~isempty(labelsPre) && ~isempty(labelsSuf));

% Determine length of trajectory
N_pre = 0;
for i = 1:length(labelsPre)
    if 0 == length(labelsPre{i}.safe)
        labelsPre{i}.len = 1;
        N_pre = N_pre + 1;   % must immediately move to next mode
    else
        labelsPre{i}.len = N;
        N_pre = N_pre + N;  % stay in mode for N-1 steps, then move
    end
end

N_suf = 0;
for i = 1:length(labelsSuf)
    if 0 == length(labelsSuf{i}.safe)
        labelsSuf{i}.len = 1;
        N_suf = N_suf + 1;   % must immediately move to next mode
    else
        labelsSuf{i}.len = N;
        N_suf = N_suf + N;  % stay in mode for N-1 steps, then move
    end
end
N = N_pre + N_suf;


% Specify system dynamics
[X, U, F_sys, sys] = getDyn(model, dt, N, ops);
assert( size(X,1) == (N+1)*sys.nx )

F_init = getInit(model, X, sys);
F = [F_sys, F_init];

% Enforce safe-reach constraints along trajectory segments
lb = 1;
for i = 1:length(labelsPre)
    trans = labelsPre{i};
    N_trans = trans.len;
    ub = lb + N_trans*sys.nx - 1;
    F_cstReach = cstReach(trans.safe, trans.reach, sys, X(lb:ub),...
                            N_trans, bigM, blkSafe);
    F = [F, F_cstReach];
    lb = ub + 1;
end

for i = 1:length(labelsSuf)
    trans = labelsSuf{i};
    N_trans = trans.len;
    ub = lb + N_trans*sys.nx - 1;
    F_cstReach = cstReach(trans.safe,trans.reach,sys,X(lb:ub),...
                            N_trans,bigM,blkSafe);
    F = [F, F_cstReach];
    lb = ub + 1;
end

% Enforce loop constraint
F = [F, getLoop(model, X(N_pre*sys.nx+1:end), sys.nx)];

alpha = 0.05;
J = alpha*norm(U(1:N_pre*sys.nu), 1) + (1-alpha)*norm(U(N_pre*sys.nu+1:end), 1);

% Solve for trajectory
sol = solvesdp(F, J, options);
traj = double(X);

% Plot the trajectory
prefix = traj(1:N_pre*sys.nx);
suffix = traj(N_pre*sys.nx+1:end);
plotTraj(prefix, suffix, model, sys, ops.dim)

end
