function [X, U, F, sys] = carDubins(dt, N, dim, linear)
% carDubins Creates sdpvars and constraints for Dubins car model
%
% Dubins car hybrid model
% x = (x,y,th,v),     u = (uk,uv) curvature (m^-1) and accel (m/s^2)
% x = position (m),   y = position (m), 
% th = orientation (rad), v = velocity (m/s)
% Dynamics: xd = v*cos(th), yd = v*sin(th), thd = uk, vd = uv
%
% Input: 'dt' timestep (sec)
%        'N' length of traj
%        'dim' workspace dimension (2D or 3D)
%        'linear' parameters for linearization of nonlinear model
% Output: 'X' sdpvar for state
%         'U' sdpvar for control
%         'F' dynamic constraints
%         'sys' system info
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

        
% Parameters
nx = 4;     % num states
nu = 2;     % num inputs 

v_min = 0.5;    % m/s
v_max = 1;      % m/s

%% Common constraints
if 3 == dim
    error('Invalid system model for 3d');
elseif 2 == dim
    sys.C = [1 0 0 0; 
             0 1 0 0];
else
    error('Invalid dimensions.');
end

% Control constraints
uk = 2*[-1,1]; % m^-1
uv = 0.25*[-1,1];   % m/s^2
Hu = [eye(nu); -eye(nu)];
Ku = [ uk(2);  uv(2); 
      -uk(1); -uv(1)];
sys.Uset = polytope(Hu,Ku);

% State constraints
x = [0,6];    % m
y = [0,6];    % m
th = pi*[-1,1];  % rad
v = [v_min,v_max];   % m/s
Hx = [eye(nx); -eye(nx)];
Kx = [ x(2);  y(2);  th(2);  v(2);
      -x(1); -y(1); -th(1); -v(1)];
sys.Xset = polytope(Hx,Kx);


%% Create system
u = sdpvar(repmat(nu,1,N),ones(1,N));
x = sdpvar(repmat(nx,1,N+1),ones(1,N+1));

F = [];
if isempty(lin)
    for t = 1:N
        F = [F, x{t+1} == x{t} + dt*[x{t}(4)*cos(x{t}(3)); 
                                     x{t}(4)*sin(x{t}(3)); 
                                     u{t}(1);
                                     u{t}(2)], ...
                Hx*x{t} <= Kx, Hu*u{t} <= Ku];
    end        
else
    % Linear hybrid model
    blk_mode = linear(1);
    numTH = linear(2);
    numV = linear(3);

    th0_vec = linspace(-pi,pi,numTH + 1) + pi/numTH;
    th0_vec = th0_vec(1:end-1)
    th_guard = linspace(-pi,pi,numTH + 1);

    v0_vec = linspace(v_min,v_max,numV)
    v_guard = linspace(v_min,v_max,numV) - (v_max-v_min)/(2*numV-2);
    v_guard(end+1) = v_guard(end) + (v_max-v_min)/(numV-1);

    A = cell(numTH,numV);
    B = cell(numTH,numV);
    K = cell(numTH,numV);
    for i = 1:numTH
        th0 = th0_vec(i);

        for j = 1:numV
            v0 = v0_vec(j);

            % Linearized model
            Ac = [0, 0, -v0*sin(th0), cos(th0); 
                  0, 0,  v0*cos(th0), sin(th0);
                  0, 0, 0, 0;
                  0, 0, 0, 0];

            Bc = [0 0; 
                  0 0; 
                  1 0; 
                  0 1];

            A{i}{j} = eye(nx) + dt*Ac;
            B{i}{j} = dt*Bc;
            K{i}{j} = dt*[ v0*th0*sin(th0);
                          -v0*th0*cos(th0);
                           0;
                           0];
        end
    end    

    M = 10*ones(nx,1);

    assert( blk_mode >= 1 )   
    if blk_mode >= 2
        fprintf('\nBlocking modes by %i.\n',blk_mode);
    end

    for t = 1:N
        if mod(t+blk_mode-1,blk_mode) == 0
            z = binvar(numTH,numV);
            F = [F, sum(z(:)) == 1];
        end
        for i=1:numTH
            for j=1:numV
                % Constrain mode based on theta and velocity values
                F = [F, x{t+1} <= A{i}{j}*x{t} + B{i}{j}*u{t} + K{i}{j} + M*(1-z(i,j)), ...
                        x{t+1} >= A{i}{j}*x{t} + B{i}{j}*u{t} + K{i}{j} - M*(1-z(i,j))];%, ...
%                             x{t}(3) <= th_guard(i+1) + M*(1 - z(i,j)), ...
%                             x{t}(3) >= th_guard(i)   - M*(1 - z(i,j)), ...
%                             x{t}(4) <= v_guard(j+1)  + M*(1 - z(i,j)), ...
%                             x{t}(4) >= v_guard(j)    - M*(1 - z(i,j))];
            end
        end

        for i=1:numTH
            % Constrain mode based on theta value
            F = [F, x{t}(3) <= th_guard(i+1) + M*(1 - sum(z(i,:))), ...
                    x{t}(3) >= th_guard(i)   - M*(1 - sum(z(i,:)))];
        end

        for j=1:numV
            % Constrain mode based on velocity value
            F = [F, x{t}(4) <= v_guard(j+1) + M*(1 - sum(z(:,j))), ...
                    x{t}(4) >= v_guard(j)   - M*(1 - sum(z(:,j)))];
        end

        F = [F, Hx*x{t}<=Kx, Hu*u{t}<= Ku];
    end
end
F = [F, Hx*x{t+1}<=Kx];

X = reshape([x{:}], (N+1)*nx,1);
U = reshape([u{:}], N*nu,1);

sys.dim = dim;
sys.nx = nx;
sys.nu = nu;

end

