function [X, U, F, sys] = quadrotor(dt, N, dim)
% quadrotor Creates sdpvars and constraints for quadrotor model
%
% Quadrotor linearized about hover (van der Burg 13)
% x = (p,v, r,w),     u = (uf,ux,uy)
% p = position (m),   v = velocity (m/s)
% r = attitude (rad), w = angular velocity (rad/s)
%
% Input: 'dt' timestep (sec)
%        'N' length of traj
%        'dim' workspace dimension (2D or 3D)
% Output: 'X' sdpvar for state
%         'U' sdpvar for control
%         'F' dynamic constraints
%         'sys' system info
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

% Parameters (modified from van der Burg 13)
g = 9.81;       % gravity (m/s^2)
m = 1;          % mass (kg)
L = 0.25;       % length of rotor arm (m)
J = 20*m*L^2;   % moment of inertia (kg m^2) (est)

tmp = [0 g; -g 0; 0 0];
Ac = [zeros(3),   eye(3),     zeros(3,2), zeros(3,2); 
      zeros(3),   zeros(3),   tmp,        zeros(3,2); 
      zeros(2,3), zeros(2,3), zeros(2),   eye(2);
      zeros(2,3), zeros(2,3), zeros(2),   zeros(2)];

tmp = [0; 0; 1/m];
Bc = [zeros(3,1) zeros(3,2); 
      tmp        zeros(3,2); 
      zeros(2,1) zeros(2,2); 
      zeros(2,1) L/J*eye(2)];


tmp = c2d(ss(Ac, Bc, [], []), dt);
sys.A = tmp.a;
sys.B = tmp.b;
sys.K = dt*zeros(10, 1);

if 3 == dim
    sys.C = [1 zeros(1, 9); 
             0 1 zeros(1, 8);
             0 0 1 zeros(1, 7)];
elseif 2 == dim
    sys.C = [1 zeros(1, 9); 
             0 1 zeros(1, 8)];
else
    error('Invalid dimension.');
end

% Control constraints
Hu = [eye(3); -eye(3)];
Ku = 0.04*[9.935; 3.62; 3.62;  
           4.545; 3.62; 3.62];
sys.Uset = polytope(Hu,Ku);

% State constraints
p = [0, 10]; % m
v = [-1, 1]; % m/s
r = [-1, 1]; % rad
w = [-1; 1]; % rad/s
Hx = [eye(10); -eye(10)];
Kx = [ p(2)*ones(3, 1);  v(2)*ones(3, 1);  r(2)*ones(2, 1);  w(2)*ones(2, 1);
      -p(1)*ones(3, 1); -v(1)*ones(3, 1); -r(1)*ones(2, 1); -w(1)*ones(2, 1)];
sys.Xset = polytope(Hx, Kx);

sys.nx = 10;    % num states
sys.nu = 3;     % num inputs
sys.dim = dim;


% Create system model
u = sdpvar(repmat(sys.nu,1,N),ones(1,N));
x = sdpvar(repmat(sys.nx,1,N+1),ones(1,N+1));

F = [];
for k = 1:N
    F = [F, x{k+1} == sys.A*x{k} + sys.B*u{k} + sys.K, ...
            Hx*x{k} <= Kx, Hu*u{k} <= Ku ];
end
F = [F, Hx*x{k+1} <= Kx ];

X = reshape([x{:}], (N+1)*sys.nx, 1);
U = reshape([u{:}], N*sys.nu, 1);
    
end