function plotTraj(prefix, suffix, model, sys, dim)
% plotTraj Plots system trajectory in workspace
%
% Input: 'prefix,' 'suffix' trajectory of system
%        'model' dynamic model
%        'sys' dynamics info
%        'dim' 2D or 3D workspace
% Output: Plot of system trajectory in workspace
%
% :copyright: 2014 by the California Institute of Technology
% :license: BSD 3-Clause, see LICENSE for details

assert( 3 == dim || 2 == dim )

prefix = reshape(prefix,sys.nx,length(prefix)/sys.nx)';
suffix = reshape(suffix,sys.nx,length(suffix)/sys.nx)';

plt_prefix = prefix*sys.C'
plt_suffix = suffix*sys.C'

xlabel('x (m)');
ylabel('y (m)');

if 3 == dim
    zlabel('z (m)');
    plot3(plt_prefix(:,1), plt_prefix(:,2), plt_prefix(:,3), ...
          'ob', 'markersize', 6, 'markerfacecolor', 'b');
    plot3(plt_suffix(:,1), plt_suffix(:,2), plt_suffix(:,3), ...
          'ok', 'markersize', 6, 'markerfacecolor', 'k');

    if strcmp(model(1:3), 'car')
        car3D(suffix);
        car3D(prefix);
    end

elseif 2 == dim
    plot(plt_prefix(:,1), plt_prefix(:,2), 'ob', 'markersize', 6, ...
         'markerfacecolor', 'b');
    plot(plt_suffix(:,1), plt_suffix(:,2), 'ok', 'markersize', 6, ...
         'markerfacecolor', 'k');

    if strcmp(model(1:3), 'car')
        car2D(suffix);
        car2D(prefix);
    end
else
    error('Invalid dimension.');
end
end


function car2D(traj)
% car2D Plots trajectory for car-like model (x,y,th) in 2D
%
% Input: 'traj' system trajectory
% Output: 2D plot of system trajectory

for i = 1:size(traj, 1)
    x0 = traj(i, 1);
    y0 = traj(i, 2);
    th0 = traj(i, 3);
    line([x0, x0 + 0.2*cos(th0)], [y0, y0 + 0.2*sin(th0)], 'color', 'b', 'linewidth', 1.5);
end
end


function car3D(traj)
% car3D Plots trajectory for car-like model (x,y,z,th) in 3D
%
% Input: 'traj' system trajectory
% Output: 3D plot of system trajectory

for i = 1:size(traj, 1)
    x0 = traj(i, 1);
    y0 = traj(i, 2);
    th0 = traj(i, 3);
    z0 = traj(i, 4);
    line([x0, x0 + 0.2*cos(th0)], [y0, y0 + 0.2*sin(th0)], [z0, z0], 'color', 'b');
end
end